@@ -13,7 +13,6 @@ from typing import (
     Dict,
     Any,
     Type,
-    ClassVar,
     Union,
     Iterable,
     Callable,
@@ -31,6 +30,7 @@ from mephisto.data_model.exceptions import (
     AgentShutdownError,
 )
 from mephisto.data_model.constants.assignment_state import AssignmentState
+import types
 
 if TYPE_CHECKING:
     from mephisto.data_model.agent import Agent, OnboardingAgent
@@ -39,7 +39,6 @@ if TYPE_CHECKING:
     from mephisto.data_model.unit import Unit
     from mephisto.data_model.packet import Packet
     from mephisto.data_model.worker import Worker
-    from argparse import _ArgumentGroup as ArgumentGroup
 
 from mephisto.operations.logger_core import get_logger
 
@@ -49,15 +48,6 @@ logger = get_logger(name=__name__)
 @dataclass
 class BlueprintArgs:
     _blueprint_type: str = MISSING
-    onboarding_qualification: str = field(
-        default=MISSING,
-        metadata={
-            "help": (
-                "Specify the name of a qualification used to block workers who fail onboarding, "
-                "Empty will skip onboarding."
-            )
-        },
-    )
     block_qualification: str = field(
         default=MISSING,
         metadata={
@@ -73,17 +63,43 @@ class SharedTaskState:
     be passed as Hydra args, like functions and objects
     """
 
-    onboarding_data: Dict[str, Any] = field(default_factory=dict)
-    task_config: Dict[str, Any] = field(default_factory=dict)
-    validate_onboarding: Callable[[Any], bool] = field(
-        default_factory=lambda: (lambda x: True)
+    task_config: Dict[str, Any] = field(
+        default_factory=dict,
+        metadata={
+            "help": (
+                "Values to be included in the frontend MephistoTask.task_config object"
+            ),
+            "type": "Dict[str, Any]",
+            "default": "{}",
+        },
+    )
+    qualifications: List[Any] = field(
+        default_factory=list,
+        metadata={
+            "help": (
+                "List of qualification dicts of the form returned by "
+                "mephisto.data_model.qualification.make_qualification_dict "
+                "to be used with this task run."
+            ),
+            "type": "List[Dict]",
+            "default": "[]",
+        },
     )
-    qualifications: List[Any] = field(default_factory=list)
     worker_can_do_unit: Callable[["Worker", "Unit"], bool] = field(
-        default_factory=lambda: (lambda worker, unit: True)
+        default_factory=lambda: (lambda worker, unit: True),
+        metadata={
+            "help": ("Function to evaluate if a worker is eligible for a given unit"),
+            "type": "Callable[[Worker, Unit], bool]",
+            "default": "Returns True always",
+        },
     )
     on_unit_submitted: Callable[["Unit"], None] = field(
-        default_factory=lambda: (lambda unit: None)
+        default_factory=lambda: (lambda unit: None),
+        metadata={
+            "help": ("Function to evaluate on every unit completed or disconnected"),
+            "type": "Callable[[Unit], None]",
+            "default": "No-op function",
+        },
     )
 
 
@@ -211,12 +227,11 @@ class TaskRunner(ABC):
             AgentShutdownError,
         ) as e:
             # A returned Unit can be worked on again by someone else.
-            if (
-                unit.get_status() != AssignmentState.EXPIRED
-                and unit.get_assigned_agent().db_id == agent.db_id
-            ):
-                logger.debug(f"Clearing {agent} from {unit} due to {e}")
-                unit.clear_assigned_agent()
+            if unit.get_status() != AssignmentState.EXPIRED:
+                unit_agent = unit.get_assigned_agent()
+                if unit_agent is not None and unit_agent.db_id == agent.db_id:
+                    logger.debug(f"Clearing {agent} from {unit} due to {e}")
+                    unit.clear_assigned_agent()
             self.cleanup_unit(unit)
         except Exception as e:
             logger.exception(f"Unhandled exception in unit {unit}: {repr(e)}")
@@ -250,7 +265,7 @@ class TaskRunner(ABC):
             AgentDisconnectedError,
             AgentShutdownError,
         ) as e:
-            # TODO(#99) if some operator flag is set for counting complete tasks, launch a
+            # TODO implement counting complete tasks, launching a
             # new assignment copied from the parameters of this one
             disconnected_agent_id = e.agent_id
             for agent in agents:
@@ -373,10 +388,9 @@ class TaskRunner(ABC):
         raise NotImplementedError()
 
 
-# TODO(#101) what is the best method for creating new ones of these for different task types
-# in ways that are supported by different backends? Perhaps abstract additional
-# methods into the required db interface? Move any file manipulations into a
-# extra_data_handler subcomponent of the MephistoDB class?
+# TODO(#567) File manipulations should ultimately be handled by the MephistoDB, rather than
+# direct reading and writing within. This allows for a better abstraction between
+# the kind of data collected and how it is stored.
 class AgentState(ABC):
     """
     Class for holding state information about work by an Agent on a Unit, currently
@@ -546,70 +560,116 @@ class AgentState(ABC):
         return 0.0
 
 
-class OnboardingRequired(object):
+class BlueprintMixin(ABC):
     """
-    Compositional class for blueprints that may have an onboarding step
+    Base class for compositional mixins for blueprints
+
+    We expect mixins that subclass other mixins to handle subinitialization
+    work, such that only the highest class needs to be called.
     """
 
+    # @property
+    # @abstractmethod
+    # def ArgsMixin(self) -> Type[object]:  # Should be a dataclass, to extend BlueprintArgs
+    #     pass
+
+    # @property
+    # @abstractmethod
+    # def SharedStateMixin(
+    #     self,
+    # ) -> Type[object]:  # Also should be a dataclass, to extend SharedTaskState
+    #     pass
+    ArgsMixin: ClassVar[Type[object]]
+    SharedStateMixin: ClassVar[Type[object]]
+
     @staticmethod
-    def get_failed_qual(qual_name: str) -> str:
-        """Returns the wrapper for a qualification to represent failing an onboarding"""
-        return qual_name + "-failed"
-
-    def init_onboarding_config(
-        self, task_run: "TaskRun", args: "DictConfig", shared_state: "SharedTaskState"
-    ):
-        self.onboarding_qualification_name: Optional[str] = args.blueprint.get(
-            "onboarding_qualification", None
+    def extract_unique_mixins(blueprint_class: Type["Blueprint"]):
+        """Return the unique mixin classes that are used in the given blueprint class"""
+        mixin_subclasses = [
+            clazz
+            for clazz in blueprint_class.mro()
+            if issubclass(clazz, BlueprintMixin)
+        ]
+        target_class: Union[Type["Blueprint"], Type["BlueprintMixin"]] = blueprint_class
+        # Remove magic created with `mixin_args_and_state`
+        while target_class.__name__ == "MixedInBlueprint":
+            target_class = mixin_subclasses.pop(0)
+        removed_locals = [
+            clazz
+            for clazz in mixin_subclasses
+            if "MixedInBlueprint" not in clazz.__name__
+        ]
+        filtered_subclasses = set(
+            clazz
+            for clazz in removed_locals
+            if clazz != BlueprintMixin and clazz != target_class
         )
-        self.onboarding_data = shared_state.onboarding_data
-        self.use_onboarding = self.onboarding_qualification_name is not None
-        self.onboarding_qualification_id = None
-        if self.onboarding_qualification_name is not None:
-            db = task_run.db
-            found_qualifications = db.find_qualifications(
-                self.onboarding_qualification_name
-            )
-            if len(found_qualifications) == 0:
-                self.onboarding_qualification_id = db.make_qualification(
-                    self.onboarding_qualification_name
-                )
-            else:
-                self.onboarding_qualification_id = found_qualifications[0].db_id
+        # we also want to make sure that we don't double-count extensions of mixins, so remove classes that other classes are subclasses of
+        def is_subclassed(clazz):
+            return True in [
+                issubclass(x, clazz) and x != clazz for x in filtered_subclasses
+            ]
 
-            # We need to keep a separate qualification for failed onboarding
-            # to push to a crowd provider in order to prevent workers
-            # who have failed from being shown our task
-            self.onboarding_failed_name = self.get_failed_qual(
-                self.onboarding_qualification_name
-            )
-            found_qualifications = db.find_qualifications(self.onboarding_failed_name)
-            if len(found_qualifications) == 0:
-                self.onboarding_failed_id = db.make_qualification(
-                    self.onboarding_failed_name
-                )
-            else:
-                self.onboarding_failed_id = found_qualifications[0].db_id
+        unique_subclasses = [
+            clazz for clazz in filtered_subclasses if not is_subclassed(clazz)
+        ]
+        return unique_subclasses
 
-    def get_onboarding_data(self, worker_id: str) -> Dict[str, Any]:
-        """
-        If the onboarding task on the frontend requires any specialized data, the blueprint
-        should provide it for the user.
+    @abstractmethod
+    def init_mixin_config(
+        self, task_run: "TaskRun", args: "DictConfig", shared_state: "SharedTaskState"
+    ) -> None:
+        """Method to initialize any required attributes to make this mixin function"""
+        raise NotImplementedError()
 
-        As onboarding qualifies a worker for all tasks from this blueprint, this should
-        generally be static data that can later be evaluated against.
-        """
-        return self.onboarding_data
+    @classmethod
+    @abstractmethod
+    def assert_mixin_args(
+        cls, args: "DictConfig", shared_state: "SharedTaskState"
+    ) -> None:
+        """Method to validate the incoming args and throw if something won't work"""
+        raise NotImplementedError()
 
-    def validate_onboarding(
-        self, worker: "Worker", onboarding_agent: "OnboardingAgent"
-    ) -> bool:
+    @classmethod
+    @abstractmethod
+    def get_mixin_qualifications(
+        cls, args: "DictConfig", shared_state: "SharedTaskState"
+    ) -> List[Dict[str, Any]]:
+        """Method to provide any required qualifications to make this mixin function"""
+        raise NotImplementedError()
+
+    @classmethod
+    def mixin_args_and_state(
+        mixin_cls: Type["BlueprintMixin"], target_cls: Type["Blueprint"]
+    ):
         """
-        Check the incoming onboarding data and evaluate if the worker
-        has passed the qualification or not. Return True if the worker
-        has qualified.
+        Magic utility decorator that can be used to inject mixin configurations
+        (BlueprintArgs and SharedTaskState) without the user needing to define new
+        classes for these. Should only be used by tasks that aren't already specifying
+        new versions of these, which should just inherit otherwise.
+
+        Usage:
+          @register_mephisto_abstraction()
+          @ABlueprintMixin.mixin_args_and_state
+          class MyBlueprint(ABlueprintMixin, Blueprint):
+              pass
         """
-        return True
+        # Ignore typing on most of this, as mypy is not able to parse what's happening
+        @dataclass
+        class MixedInArgsClass(mixin_cls.ArgsMixin, target_cls.ArgsClass):  # type: ignore
+            pass
+
+        @dataclass
+        class MixedInSharedStateClass(
+            mixin_cls.SharedStateMixin, target_cls.SharedStateClass  # type: ignore
+        ):
+            pass
+
+        class MixedInBlueprint(target_cls):  # type: ignore
+            ArgsClass = MixedInArgsClass
+            SharedStateClass = MixedInSharedStateClass
+
+        return MixedInBlueprint
 
 
 class Blueprint(ABC):
@@ -637,6 +697,20 @@ class Blueprint(ABC):
         self.shared_state = shared_state
         self.frontend_task_config = shared_state.task_config
 
+        # We automatically call all mixins `init_mixin_config` methods available.
+        mixin_subclasses = BlueprintMixin.extract_unique_mixins(self.__class__)
+        for clazz in mixin_subclasses:
+            clazz.init_mixin_config(self, task_run, args, shared_state)
+
+    @classmethod
+    def get_required_qualifications(
+        cls, args: DictConfig, shared_state: "SharedTaskState"
+    ):
+        quals = []
+        for clazz in BlueprintMixin.extract_unique_mixins(cls):
+            quals += clazz.get_mixin_qualifications(args, shared_state)
+        return quals
+
     @classmethod
     def assert_task_args(cls, args: DictConfig, shared_state: "SharedTaskState"):
         """
@@ -644,6 +718,10 @@ class Blueprint(ABC):
         fail if a task launched with these arguments would
         not work
         """
+        # We automatically call all mixins `assert_task_args` methods available.
+        mixin_subclasses = BlueprintMixin.extract_unique_mixins(cls)
+        for clazz in mixin_subclasses:
+            clazz.assert_mixin_args(args, shared_state)
         return
 
     def get_frontend_args(self) -> Dict[str, Any]:
@@ -651,7 +729,7 @@ class Blueprint(ABC):
         Specifies what options should be fowarded
         to the client for use by the task's frontend
         """
-        return self.frontend_task_config
+        return self.frontend_task_config.copy()
 
     @abstractmethod
     def get_initialization_data(
