@@ -27,8 +27,20 @@ from mephisto.data_model.packet import (
 from mephisto.data_model.worker import Worker
 from mephisto.data_model.qualification import worker_is_qualified
 from mephisto.data_model.agent import Agent, OnboardingAgent
-from mephisto.abstractions.blueprint import OnboardingRequired, AgentState
+from mephisto.abstractions.blueprint import AgentState
+from mephisto.abstractions.blueprints.mixins.onboarding_required import (
+    OnboardingRequired,
+)
+from mephisto.abstractions.blueprints.mixins.screen_task_required import (
+    ScreenTaskRequired,
+)
+from mephisto.abstractions.blueprints.mixins.use_gold_unit import UseGoldUnit
 from mephisto.operations.registry import get_crowd_provider_from_type
+from mephisto.operations.task_launcher import (
+    TaskLauncher,
+    SCREENING_UNIT_INDEX,
+    GOLD_UNIT_INDEX,
+)
 from mephisto.abstractions.channel import Channel, STATUS_CHECK_TIME
 
 from dataclasses import dataclass
@@ -79,6 +91,7 @@ class Job:
     provider: "CrowdProvider"
     qualifications: List[Dict[str, Any]]
     registered_channel_ids: List[str]
+    task_launcher: Optional["TaskLauncher"]
 
 
 @dataclass
@@ -155,6 +168,7 @@ class Supervisor:
             provider=provider,
             qualifications=qualifications,
             registered_channel_ids=[],
+            task_launcher=None,
         )
         for channel in channels:
             channel_id = self.register_channel(channel, job)
@@ -381,8 +395,8 @@ class Supervisor:
         self, unit: "Unit", agent_info: "AgentInfo", task_runner: "TaskRunner"
     ):
         """Launch a thread to supervise the completion of an assignment"""
+        agent = agent_info.agent
         try:
-            agent = agent_info.agent
             assert isinstance(
                 agent, Agent
             ), f"Can launch units for Agents, not OnboardingAgents, got {agent}"
@@ -399,6 +413,15 @@ class Supervisor:
             logger.exception(f"Cleaning up unit: {e}", exc_info=True)
             task_runner.cleanup_unit(unit)
         finally:
+            if unit.unit_index in [SCREENING_UNIT_INDEX, GOLD_UNIT_INDEX]:
+                if agent.get_status() != AgentState.STATUS_COMPLETED:
+                    if unit.unit_index == SCREENING_UNIT_INDEX:
+                        blueprint = task_runner.task_run.get_blueprint(
+                            args=task_runner.args
+                        )
+                        assert isinstance(blueprint, ScreenTaskRequired)
+                        blueprint.screening_units_launched -= 1
+                    unit.expire()
             task_runner.task_run.clear_reservation(unit)
 
     def _assign_unit_to_agent(
@@ -452,7 +475,7 @@ class Supervisor:
             ] = agent_info
 
             # Launch individual tasks
-            if not channel_info.job.task_runner.is_concurrent:
+            if unit.unit_index < 0 or not channel_info.job.task_runner.is_concurrent:
                 unit_thread = threading.Thread(
                     target=self._launch_and_run_unit,
                     args=(unit, agent_info, channel_info.job.task_runner),
@@ -510,6 +533,7 @@ class Supervisor:
             isinstance(blueprint, OnboardingRequired) and blueprint.use_onboarding
         ), "Should only be registering from onboarding if onboarding is required and set"
         worker_passed = blueprint.validate_onboarding(worker, onboarding_agent)
+        assert blueprint.onboarding_qualification_name is not None
         worker.grant_qualification(
             blueprint.onboarding_qualification_name, int(worker_passed)
         )
@@ -589,8 +613,7 @@ class Supervisor:
                 )
             )
             logger.debug(
-                f"Found existing agent_registration_id {agent_registration_id}, "
-                f"had no valid units."
+                f"agent_registration_id {agent_registration_id}, had no valid units."
             )
             return
 
@@ -657,6 +680,54 @@ class Supervisor:
                 agent_info.assignment_thread = onboard_thread
                 onboard_thread.start()
                 return
+        if isinstance(blueprint, ScreenTaskRequired) and blueprint.use_screening_task:
+            if (
+                blueprint.worker_needs_screening(worker)
+                and blueprint.should_generate_unit()
+            ):
+                screening_data = blueprint.get_screening_unit_data()
+                if screening_data is not None:
+                    launcher = channel_info.job.task_launcher
+                    assert (
+                        launcher is not None
+                    ), "Job must have launcher to use screening tasks"
+                    units = [launcher.launch_screening_unit(screening_data)]
+                else:
+                    self.message_queue.append(
+                        Packet(
+                            packet_type=PACKET_TYPE_PROVIDER_DETAILS,
+                            sender_id=SYSTEM_CHANNEL_ID,
+                            receiver_id=channel_info.channel_id,
+                            data={
+                                "request_id": packet.data["request_id"],
+                                "agent_id": None,
+                            },
+                        )
+                    )
+                    logger.debug(
+                        f"No screening units left for {agent_registration_id}."
+                    )
+                    return
+        if isinstance(blueprint, UseGoldUnit) and blueprint.use_golds:
+            if blueprint.should_produce_gold_for_worker(worker):
+                gold_data = blueprint.get_gold_unit_data_for_worker(worker)
+                if gold_data is not None:
+                    launcher = channel_info.job.task_launcher
+                    units = [launcher.launch_gold_unit(gold_data)]
+                else:
+                    self.message_queue.append(
+                        Packet(
+                            packet_type=PACKET_TYPE_PROVIDER_DETAILS,
+                            sender_id=SYSTEM_CHANNEL_ID,
+                            receiver_id=channel_info.channel_id,
+                            data={
+                                "request_id": packet.data["request_id"],
+                                "agent_id": None,
+                            },
+                        )
+                    )
+                    logger.debug(f"No gold units left for {agent_registration_id}...")
+                    return
 
         # Not onboarding, so just register directly
         self._assign_unit_to_agent(packet, channel_info, units)
